function stationaryDistribution = getxDist(P,p,xGrid,x0,xNextMat,zeta,xNextSlope,h)
% compute stationary distribution of state variable x by Pareto extrapolation
% P: transition probability matrix of Markov chain
% p: birth/death probability (set 0 for infinitely-lived case)
% xGrid: grid for state variable x (asset, wealth, etc.)
% x0: initial state variable of newborn agents (not used if p=0)
% xNextMat: matrix of next period's state on each point of xGrid
% zeta: Pareto exponent
% xNextSlope: slope of law of motion of x at largest grid (optional)
% h: grid spacing for extrapolation (optional)

%% some error checking

if any(diff(xGrid) <= 0)
    error('xGrid must be in strictly increasing order')
end

[S,N] = size(xNextMat); % number of exogenous states and grid points for x

if length(xGrid) ~= N
    error('xGrid and xNextMat must be consistent')
end

if N < 2
    error('xGrid must contain at least two elements')
end

if p>0 % there is birth/death
    if x0 < xGrid(1)
        error('it must be x0 >= min(xGrid)')
    elseif x0 >= xGrid(end)
        error('it must be x0 < max(xGrid)')
    end   
end

%% define optional arguments if not provided

if nargin < 6 % Pareto exponent not provided, just use usual method
    flag = 0;
elseif zeta <= 1
    error('zeta must be larger than 1')
else
    flag = 1; % use Pareto extrapolation
end

xMax = max(xGrid);
if xMax <= 0
    warning('largest grid point is negative')
    flag = 0; % if largest grid point negative, Pareto extrapolation not defined
end

if nargin < 7 % slope of law of motion not provided, just use linear extrapolation
    xNextSlope = (xNextMat(:,end) - xNextMat(:,end-1))/(xGrid(end) - xGrid(end-1));
end

if nargin < 8 % grid spacing for extrapolation not provided
    h = (xGrid(end) - xGrid(end-1)); % use distance between largest two grid points
end

%% construct hypothetical extra grid points and law of motion if Pareto extrapolation is used
if flag == 1 % use Pareto extrapolation
    xlim = xMax + (xMax - xNextMat(:,end))./xNextSlope; % values of x that make extrapolating law of motion hit xMax
    if max(xlim) <= xMax % no need to extrapolate
        flag = 0; % switch back to usual method
    else % need to extrapolate
        Nextra = ceil((max(xlim) - xMax)/h); % number of extra hypothetical grid points needed (excluding xMax)
        %Nextra = min(Nextra,1000); % limit extra grid points to a reasonable number
        ParetoCDF = (1 + Nextra*h/xMax)^(-zeta); % conditional Pareto CDF at largest extra grid point, conditional on x > xMax
        ParetoPDF = (zeta/xMax)*(1 + [0:Nextra]*h/xMax).^(-zeta-1); % conditional Pareto PDF at extra grid points (including xMax)
        qtail = ParetoPDF*h; % adjust for grid spacing (trapezoidal formula)
        qtail(end) = ParetoCDF;
        qtail = qtail/sum(qtail); % conditional probability on extra grid points
        xNextExtra = bsxfun(@plus,xNextMat(:,end),h*xNextSlope*[0:Nextra]); % law of motion on extra grid points
    end
end

%% construct the combined transition probability matrix

Q = zeros(N*S); % transition probability matrix of (s,x)

% take care of newborn agents
Qx0 = zeros(N);
if p>0
    ind0 = find(xGrid < x0, 1, 'last');
    if isempty(ind0) % x0 is below smallest grid point
        Qx0(:,1) = 1;
    elseif ind0 < N % x0 is strictly inside grid
        t0 = (x0-xGrid(ind0))/(xGrid(ind0+1)-xGrid(ind0));
        Qx0(:,ind0) = 1-t0; % convex combination
        Qx0(:,ind0+1) = t0;
    else % x0 is above largest grid
        Qx0(:,end) = 1;
    end
end

for s = 1:S
    Qx = zeros(N); % transition probability matrix on xGrid, conditional on s
    % first, construct probability matrix using Truncation method
    for n = 1:N
        x = xNextMat(s,n);
        ind = find(xGrid < x, 1, 'last' );
        if isempty(ind) % x is below smallest grid point
            Qx(n,1) = 1; % put all mass to smallest grid point
        elseif ind < N % x is strictly inside grid
            t = (x-xGrid(ind))/(xGrid(ind+1)-xGrid(ind));
            Qx(n,ind) = 1-t; % convex combination
            Qx(n,ind+1) = t;
        else % x is above largest grid point
            Qx(n,N) = 1; % put all mass on largest grid point
        end
    end
    if flag == 1 % use Pareto extrapolation to correct transition probability
        Qx(N,:) = 0; % last row of Qx is incorrect, reset to 0
        for n = 1:Nextra+1 % iterate over hypothetical extra grid points
            x = xNextExtra(s,n);
            ind = find(xGrid < x, 1, 'last' );
            q = qtail(n);
            if isempty(ind) % x is below smallest grid point
                Qx(N,1) = Qx(N,1) + q; % put all mass to smallest grid point
            elseif ind < N % x is inside grid
                t = (x-xGrid(ind))/(xGrid(ind+1)-xGrid(ind));
                Qx(N,ind) = Qx(N,ind) + (1-t)*q; % convex combination
                Qx(N,ind+1) = Qx(N,ind+1) + t*q;
            else  % x is above largest grid point
                Qx(N,N) = Qx(N,N) + q; % put all mass on largest grid point
            end
        end
    end
    Qx = p*Qx0 + (1-p)*Qx; % take care of newborn agents
    Q(N*(s-1)+1:N*s,:) = kron(P(s,:),Qx); % construct the big matrix using exogenous state
end

[v,~] = eigs(Q',1,1);
stationaryDistribution = v/sum(v);

end

